@@ -228,20 +228,21 @@ class Registry(Mapping[URI, Resource[D]]):
     even according to the retrieval logic.
     """
 
-    _resources: HashTrieMap[URI, Resource[D]] = field(  # type: ignore[reportGeneralTypeIssues]  # noqa: E501
+    _resources: HashTrieMap[URI, Resource[D]] = field(
         default=HashTrieMap(),
-        converter=HashTrieMap.convert,
+        converter=HashTrieMap.convert,  # type: ignore[reportGeneralTypeIssues]  # noqa: E501
+        alias="resources",
     )
     _anchors: HashTrieMap[tuple[URI, str], AnchorType[D]] = HashTrieMap()  # type: ignore[reportGeneralTypeIssues]  # noqa: E501
     _uncrawled: HashTrieSet[URI] = EMPTY_UNCRAWLED
-    _retrieve: Retrieve[D] = field(default=_fail_to_retrieve)
+    _retrieve: Retrieve[D] = field(default=_fail_to_retrieve, alias="retrieve")
 
     def __getitem__(self, uri: URI) -> Resource[D]:
         """
         Return the (already crawled) `Resource` identified by the given URI.
         """
         try:
-            return self._resources[uri]
+            return self._resources[uri.rstrip("#")]
         except KeyError:
             raise exceptions.NoSuchResource(ref=uri)
 
@@ -364,27 +365,29 @@ class Registry(Mapping[URI, Resource[D]]):
         """
         Retrieve a given anchor from a resource which must already be crawled.
         """
-        resource = self.get(uri)
-        if resource is None:
-            canonical_uri = uri
-        else:
-            canonical_uri = resource.id() or uri
-
-        value = self._anchors.get((canonical_uri, name))
+        value = self._anchors.get((uri, name))
         if value is not None:
             return Retrieved(value=value, registry=self)
 
         registry = self.crawl()
-        value = registry._anchors.get((canonical_uri, name))
+        value = registry._anchors.get((uri, name))
         if value is not None:
             return Retrieved(value=value, registry=registry)
+
+        resource = self[uri]
+        canonical_uri = resource.id()
+        if canonical_uri is not None:
+            value = registry._anchors.get((canonical_uri, name))
+            if value is not None:
+                return Retrieved(value=value, registry=registry)
+
         if "/" in name:
             raise exceptions.InvalidAnchor(
                 ref=uri,
-                resource=self[uri],
+                resource=resource,
                 anchor=name,
             )
-        raise exceptions.NoSuchAnchor(ref=uri, resource=self[uri], anchor=name)
+        raise exceptions.NoSuchAnchor(ref=uri, resource=resource, anchor=name)
 
     def contents(self, uri: URI) -> D:
         """
@@ -502,16 +505,16 @@ class Registry(Mapping[URI, Resource[D]]):
 
 
 #: An anchor or resource.
-T = TypeVar("T", AnchorType[Any], Resource[Any])
+AnchorOrResource = TypeVar("AnchorOrResource", AnchorType[Any], Resource[Any])
 
 
 @frozen
-class Retrieved(Generic[D, T]):
+class Retrieved(Generic[D, AnchorOrResource]):
     """
     A value retrieved from a `Registry`.
     """
 
-    value: T
+    value: AnchorOrResource
     registry: Registry[D]
 
 
@@ -533,6 +536,12 @@ class Resolver(Generic[D]):
     Resolvers help resolve references (including relative ones) by
     pairing a fixed base URI with a `Registry`.
 
+    This object, under normal circumstances, is expected to be used by
+    *implementers of libraries* built on top of `referencing` (e.g. JSON Schema
+    implementations or other libraries resolving JSON references),
+    not directly by end-users populating registries or while writing
+    schemas or other resources.
+
     References are resolved against the base URI, and the combined URI
     is then looked up within the registry.
 
